use std::str::FromStr;

use rocket::{
	http::{CookieJar, Status},
	request::{FromRequest, Outcome},
	response::Redirect,
	Request,
};
use uuid::Uuid;

use crate::db::{
	schema::{DollTagsDb, User},
	user,
};

pub fn login(cookies: &CookieJar<'_>, user_id: &Uuid) {
	cookies.add_private(("user_id", user_id.to_string()));
}

pub fn check_login(cookies: &CookieJar<'_>) -> Option<Uuid> {
	cookies
		.get_private("user_id")
		.and_then(|v| Uuid::from_str(&v.value()).ok())
}

pub fn logout(cookies: &CookieJar<'_>) {
	cookies.remove_private("user_id");
}

#[derive(Debug)]
pub struct SessionInternalFailure();

#[rocket::async_trait]
impl<'a> FromRequest<'a> for User {
	type Error = SessionInternalFailure;

	async fn from_request(req: &'a Request<'_>) -> Outcome<Self, Self::Error> {
		let cookies = req.cookies();

		if let Some(id) = check_login(&cookies) {
			let db = DollTagsDb::from_request(req)
				.await
				.expect("User::from_request cannot get DB connection");

			match user::get_by_id(db, &id).await {
				Err(err) => {
					error!("User::from_request internal error: {:?}", err);
					Outcome::Error((Status::InternalServerError, SessionInternalFailure()))
				}
				Ok(Some(user)) => Outcome::Success(user),
				Ok(None) => Outcome::Forward(Status::Unauthorized),
			}
		} else {
			Outcome::Forward(Status::Unauthorized)
		}
	}
}

#[catch(401)]
pub fn unauthorized(req: &Request) -> Redirect {
	let next = req.uri().to_string();
	Redirect::to(uri!(crate::routes::form::accounts::show_login(Some(&next))))
}
